@@ -3,13 +3,9 @@
 #include "json.h"
 #define FIO_DU_NAME_SZ		64
 
-extern volatile int disk_util_exit;
+extern volatile int helper_exit;
 
-/*
- * Disk utils as read in /sys/block/<dev>/stat
- */
-struct disk_util_stat {
-	uint8_t name[FIO_DU_NAME_SZ];
+struct disk_util_stats {
 	uint32_t ios[2];
 	uint32_t merges[2];
 	uint64_t sectors[2];
@@ -19,6 +15,14 @@ struct disk_util_stat {
 	uint64_t msec;
 };
 
+/*
+ * Disk utils as read in /sys/block/<dev>/stat
+ */
+struct disk_util_stat {
+	uint8_t name[FIO_DU_NAME_SZ];
+	struct disk_util_stats s;
+};
+
 struct disk_util_agg {
 	uint32_t ios[2];
 	uint32_t merges[2];
@@ -96,8 +100,6 @@ static inline void disk_util_dec(struct disk_util *du)
 
 extern struct flist_head disk_list;
 
-extern void wait_for_disk_thread_exit(void);
-
 /*
  * disk util stuff
  */
@@ -123,12 +125,8 @@ static inline void print_disk_util(struct disk_util_stat *du,
 
 static inline int update_io_ticks(void)
 {
-	return disk_util_exit;
+	return helper_exit;
 }
 #endif
 
-static inline void disk_util_start_exit(void)
-{
-	disk_util_exit = 1;
-}
 #endif
