@@ -69,6 +69,33 @@ static int fio_io_end(struct thread_data *td, struct io_u *io_u, int ret)
 	return FIO_Q_COMPLETED;
 }
 
+#ifdef CONFIG_PWRITEV
+static int fio_pvsyncio_queue(struct thread_data *td, struct io_u *io_u)
+{
+	struct syncio_data *sd = td->io_ops->data;
+	struct iovec *iov = &sd->iovecs[0];
+	struct fio_file *f = io_u->file;
+	int ret;
+
+	fio_ro_check(td, io_u);
+
+	iov->iov_base = io_u->xfer_buf;
+	iov->iov_len = io_u->xfer_buflen;
+
+	if (io_u->ddir == DDIR_READ)
+		ret = preadv(f->fd, iov, 1, io_u->offset);
+	else if (io_u->ddir == DDIR_WRITE)
+		ret = pwritev(f->fd, iov, 1, io_u->offset);
+	else if (io_u->ddir == DDIR_TRIM) {
+		do_io_u_trim(td, io_u);
+		return FIO_Q_COMPLETED;
+	} else
+		ret = do_io_u_sync(td, io_u);
+
+	return fio_io_end(td, io_u, ret);
+}
+#endif
+
 static int fio_psyncio_queue(struct thread_data *td, struct io_u *io_u)
 {
 	struct fio_file *f = io_u->file;
@@ -111,7 +138,7 @@ static int fio_syncio_queue(struct thread_data *td, struct io_u *io_u)
 
 static int fio_vsyncio_getevents(struct thread_data *td, unsigned int min,
 				 unsigned int max,
-				 struct timespec fio_unused *t)
+				 const struct timespec fio_unused *t)
 {
 	struct syncio_data *sd = td->io_ops->data;
 	int ret;
@@ -329,11 +356,28 @@ static struct ioengine_ops ioengine_vrw = {
 	.flags		= FIO_SYNCIO,
 };
 
+#ifdef CONFIG_PWRITEV
+static struct ioengine_ops ioengine_pvrw = {
+	.name		= "pvsync",
+	.version	= FIO_IOOPS_VERSION,
+	.init		= fio_vsyncio_init,
+	.cleanup	= fio_vsyncio_cleanup,
+	.queue		= fio_pvsyncio_queue,
+	.open_file	= generic_open_file,
+	.close_file	= generic_close_file,
+	.get_file_size	= generic_get_file_size,
+	.flags		= FIO_SYNCIO,
+};
+#endif
+
 static void fio_init fio_syncio_register(void)
 {
 	register_ioengine(&ioengine_rw);
 	register_ioengine(&ioengine_prw);
 	register_ioengine(&ioengine_vrw);
+#ifdef CONFIG_PWRITEV
+	register_ioengine(&ioengine_pvrw);
+#endif
 }
 
 static void fio_exit fio_syncio_unregister(void)
@@ -341,4 +385,7 @@ static void fio_exit fio_syncio_unregister(void)
 	unregister_ioengine(&ioengine_rw);
 	unregister_ioengine(&ioengine_prw);
 	unregister_ioengine(&ioengine_vrw);
+#ifdef CONFIG_PWRITEV
+	unregister_ioengine(&ioengine_pvrw);
+#endif
 }
