@@ -173,7 +173,7 @@ void prune_io_piece_log(struct thread_data *td)
 	}
 
 	while (!flist_empty(&td->io_hist_list)) {
-		ipo = flist_entry(&td->io_hist_list, struct io_piece, list);
+		ipo = flist_first_entry(&td->io_hist_list, struct io_piece, list);
 		flist_del(&ipo->list);
 		remove_trim_entry(td, ipo);
 		td->io_hist_len--;
@@ -871,18 +871,18 @@ int iolog_file_inflate(const char *file)
 	 */
 	total = ic.len;
 	do {
-		size_t ret;
+		size_t iret;
 
-		ret = inflate_chunk(&ic,  1, stdout, &stream, &iter);
-		total -= ret;
+		iret = inflate_chunk(&ic,  1, stdout, &stream, &iter);
+		total -= iret;
 		if (!total)
 			break;
 		if (iter.err)
 			break;
 
 		ic.seq++;
-		ic.len -= ret;
-		ic.buf += ret;
+		ic.len -= iret;
+		ic.buf += iret;
 	} while (1);
 
 	if (iter.seq) {
